# Code for the non-GUI RoutePlanner
#
# Copyright 1996-2000 Chris Lawrence
# This file may be freely distributed under the terms of the RoutePlanner
# license.  A copy should appear as 'LICENSE' in the archive that this
# file was included in.

import rpdbase, string, sys, os

# If readline works, import it; else, don't worry about it
try:
    import readline
except:
    pass

def ChooseCity(prompt, db):
    while 1:
        cities = []
        while not cities:
            try:
                stuff = raw_input(prompt+'\n> ')
            except EOFError:
                return None

            if not stuff:
                return None
            cities = db.CitiesMatching(stuff)
            if not cities:
                print 'No cities matching '+stuff
                
            if len(cities) == 1:
                return cities[0]
            
            while cities:
                count = 1
                for city in cities:
                    print '%2d: %s' % (count, city)
                    count = count + 1
                    if count > 15:
                        print '(More...)'
                        break
                try:
                    stuff = raw_input('Choose a city (by number or name)\n> ')
                except EOFError:
                    return None
                if not stuff:
                    return None
                
                try:
                    num = int(stuff)
                except:
                    num = 0

                if 0 < num < count:
                    return cities[num-1]
                elif num:
                    print 'Invalid choice (must be between 1 and %d).' % \
                          (count - 1)
                    continue
                else:
                    cities = db.CitiesMatching(stuff)
                    if not cities:
                        print 'No cities matching '+stuff
                        continue
                    elif len(cities) == 1:
                        return cities[0]
# End of ChooseCity

def NicerTime(min):
    return '%2d:%02d' % (min / 60, min % 60)

def mainloop(database='Basic-USA.rpl3'):
    print rpdbase.VERSION
    print rpdbase.COPYRIGHT

    units = rpdbase.units
    if database and not os.path.exists(database):
        database = '/usr/share/routeplanner/'+database

    if database:
        print 'Loading database '+database+'...',
        db = rpdbase.RPDatabase(database)
        units = db.units
        print 'Done'
    else:
        print 'No database specified.'
        return 1
    print '%d cities, %d routes' % (len(db.cityhash), len(db.routes))
    print 'Created by %s <%s>' % (db.author, db.author_email)
    print
    
    while 1:
        city = ChooseCity('Please enter the name of the city you want '
                          'to start at:', db)
        if not city:
            return 1
        print 'Selected: '+str(city)
        
        path = [city]
        while city:
            city = ChooseCity('Enter destination for next leg (or an empty '
                              'destination to plan route):', db)
            if city:
                print 'Selected: '+str(city)
                path.append(city)
        
        if len(path) < 2:
            print 'You really need to specify two or more cities.'
            continue

        method = 0
        while method == 0:
            print 'Choose routing method:'
            for i in range(len(rpdbase.methods)):
                print '%2d: %s' % (i+1, rpdbase.methods[i])

            junk = raw_input('> ')
            try:
                method = int(junk)
            except:
                pass
            if not (1 <= method <= len(rpdbase.methods)):
                method = 0
        
        method = method - 1
        print 'Planning route ('+rpdbase.methods[method]+'):'
        for entry in path:
            print '  '+str(entry)

        import timing
##        print "Running old Navigate()...",
##        timing.start()
##        trail = []
##        for i in xrange(len(path)-1):
##            trail.append( db.OldNavigate(path[i], path[i+1], method=method) )
##        timing.finish()
##        print "done in %d ms." % timing.milli()
        
        print "Running new Navigate()...",
        sys.stdout.flush()
        trail = []
        timing.start()
        for i in xrange(len(path)-1):
            trail.append( db.NewNavigate(path[i], path[i+1], method=method) )
        timing.finish()
        print "done in %d ms." % timing.milli()
        
        displist = []
        
        for leg in trail:
            tlist = []
            entry = None
            for segment in leg:
                preventry = entry
                entry = ( segment[0].city[1-segment[1]],
                          segment[0].city[segment[1]],
                          segment[0].name,
                          segment[0].exits[segment[1]],
                          rpdbase.ConvertDistance(segment[0].distance,
                                                  segment[0].units, units),
                          rpdbase.ConvertToTime(segment[0].speedcode,
                                                segment[0].speed,
                                                segment[0].distance,
                                                segment[0].units,
                                                db.units, db.speed),
                          rpdbase.Efficiency(segment[0].distance,
                                             segment[0].units,
                                             db.units, units,
                                             segment[0].speedcode,
                                             db.mpg, db.defmpg) )
                if preventry:
                    if rpdbase.IsSameRoute(preventry[2], entry[2]):
                        del tlist[len(tlist)-1]
                        entry = (preventry[0], entry[1], preventry[2],
                                 entry[3], entry[4]+preventry[4],
                                 entry[5]+preventry[5], entry[6]+preventry[6])

                tlist.append(entry)

            displist.append(tlist)
            
        del trail # Free some memory
        
        # Display our route
        if units == 'km':
            mpgunits = 'l'
        else:
            mpgunits = 'gal'

        totals = (0,0,0)
        for i in xrange(len(displist)):
            print 'From '+str(path[i])+' to '+str(path[i+1])
            print '-'*60
            
            leg = (0,0,0)
            for segment in displist[i]:
                print NicerTime(segment[5]) + ' ' + str(segment[2]) + ' to ' +\
                      str(segment[1]),
                if segment[3]:
                    print '[Exit '+segment[3]+']',
                print '(%.0f %s)' % (segment[4], units)
                leg = leg[0] + segment[5], leg[1] + segment[4],\
                      leg[2] + segment[6]

            if len(displist) > 1:
                print 'Leg totals: %.0f %s; %s (%.1f %s)' % (leg[1], units,
                                                             NicerTime(leg[0]),
                                                             leg[2], mpgunits)
                print

            totals = (totals[0]+leg[0], totals[1]+leg[1], totals[2]+leg[2])

        print 'Total route: %.0f %s; %s (%.1f %s)' % (totals[1], units,
                                                      NicerTime(totals[0]),
                                                      totals[2], mpgunits)
        # We is done

    # End of while
# End of mainloop
