# Code for the GUI interface to RoutePlanner
#
# Copyright (C) 1996-2000 Chris Lawrence
# This file may be freely distributed under the terms of the RoutePlanner
# license.  A copy should appear as 'LICENSE' in the archive that this
# file was included in.

import rpdbase, string, sys, os, glob, commands, time, libglade, gnome, GDK

gnome.app_id = 'RoutePlanner'
gnome.app_version = rpdbase.VERID

from gtk import *
from gnome.ui import *
from GDK import *

def NicerTime(min):
    return '%2d:%02d' % (min / 60, min % 60)

PANES = ['shortroute', 'quickroute', 'prefroute']
GUIFILE = '/usr/share/routeplanner/routeplanner.glade'

class ProgressWin:
    def __init__(self, wtree, title):
        self.wtree = wtree
        self.current, self.max = 0, 1
        self.window = wtree.get_widget("progresswindow")
        self.bar = wtree.get_widget("progressbar1")
        self.bartitle = wtree.get_widget("progresslabel")
        self.window.set_title(title)
        self.window.connect("delete_event", self.close_clicked)

    def set_label(self, label):
        self.bartitle.set_text(label)

        while events_pending():
            mainiteration(FALSE)

    def set_max(self, maxval):
        self.max = maxval
        self.bar.update(self.current/float(self.max))

        while events_pending():
            mainiteration(FALSE)

    def set_current(self, current):
        self.current = current
        if self.max:
            self.bar.update(current/float(self.max))
        else:
            self.bar.update(0)

        while events_pending():
            mainiteration(FALSE)

    def close_clicked(self, *args):
        self.hide()
        while events_pending():
            mainiteration(FALSE)

        return TRUE

    def hide(self):
        while events_pending():
            mainiteration(FALSE)
        self.window.hide()

    def show(self):
        self.window.show()
        while events_pending():
            mainiteration(FALSE)

class CityList:
    def __init__(self, wtree):
        self.wtree = wtree
        self.citywin = wtree.get_widget('choosecity')
        self.citylist = wtree.get_widget('citychooser')
        self.cityentry = wtree.get_widget('cityentry')
        self.showallbutton = wtree.get_widget('showall')
        self.visible = []

        dict = {}
        for key in dir(self.__class__):
            dict[key] = getattr(self, key)
        self.wtree.signal_autoconnect(dict)

    def startup(self, city='', db=None, setfun=None, setdata=None):
        self.db = db
        self.cities = map(str, db.cities)

        # Update the CList
        self.update_citylist()
        if city and city in self.cities:
            cnum = self.cities.index(city)
            self.citylist.select_row(cnum, 0)
            self.selected = city
            self.selrow = cnum
        else:
            self.selected = self.selrow = None
            self.cityentry.set_text('')
        
        self.citywin.connect("delete_event", self.on_city_cancel_clicked)
        
        if setfun:
            self.callwhendone = (setfun, setdata)
        else:
            self.callwhendone = None
            
        self.citywin.show()

    def on_citychooser_select_row(self, *args):
        pos = self.visible[args[1]]
        self.selected = self.cities[pos]
        self.selrow = pos
        self.cityentry.set_text(self.selected)

    def on_citychooser_unselect_row(self, *args):
        self.selected = self.selrow = None
        self.cityentry.set_text('')

    def on_citychooser_button_press_event(self, button, event, *args):
        if event.type == GDK._2BUTTON_PRESS and event.button == 1:
            self.on_city_ok_clicked()

    def on_cityentry_activate(self, *args):
        text = self.cityentry.get_text()
        matching = self.db.CitiesMatchIndex(text, self.visible)
        if not matching:
            dialog = GnomeWarningDialog('No matching cities', self.citywin)
            dialog.run_and_close()
            return

        if len(matching) == 1:
            match, lmatch = matching[0], self.visible.index(matching[0])
            self.citylist.select_row(lmatch, 0)
            if string.lower(text) == string.lower(self.cities[match]):
                self.on_city_ok_clicked()
            return

        self.update_citylist(tuple(matching))

    def on_city_ok_clicked(self, *args):
        if not self.selected:
            self.on_cityentry_activate()
            return

        self.citywin.hide()
        self.citylist.clear()
        if self.callwhendone:
            setfun, setdata = self.callwhendone
            setfun(self.selected, self.selrow, setdata)

    def on_city_cancel_clicked(self, *args):
        self.citywin.hide()
        self.citylist.clear()
        return TRUE

    def on_city_showall_clicked(self, *args):
        self.update_citylist()

    def update_citylist(self, selected=()):
        if not selected:
            selected = range(len(self.cities))

        self.citylist.freeze()
        self.citylist.clear()

        for i in range(len(selected)):
            sel = selected[i]
            self.citylist.append([self.cities[sel]])
            
        self.visible = list(selected)
        self.citylist.thaw()
        self.showallbutton.set_sensitive(len(selected) != len(self.cities))

class Application:
    def __init__(self, filename=''):
        if os.path.exists('routeplanner.glade'):
            self.wtree = libglade.GladeXML('routeplanner.glade')
        else:
            self.wtree = libglade.GladeXML(GUIFILE)
        
        self.appwin = self.wtree.get_widget('app1')
        self.appwin.connect("destroy", mainquit) 
##        print "Instance self.wtree",self.wtree
##        print "\n***Dictionary ot the instance were I can see self.wtree"
##        print(self.__dict__)
##        print "\nDictionary of the class were I can see on_about_cb"
##        print(Application.__dict__)
        # You need to create this diccionary in order to get the 
        # the bound methods that you can autoconnect with.
        # And quoting James: 
        # "I know it is not perfect, but it should do what you want."
        dict = {}
        for key in dir(self.__class__):
            dict[key] = getattr(self, key)
        self.wtree.signal_autoconnect(dict)
        self.db = None
        self.citylist = self.wtree.get_widget('citylist')
        self.citywin = None
        self.make_widgets_sensitive()
        # Hardcoded for 0.1
        self.units = 'mi'
        if filename:
            self.open_database(filename)

    def on_exit1_activate(self,*args):
        mainquit()

    def on_about1_activate(self,*args):
        about = self.wtree.get_widget('aboutbox')
        about.close_hides(TRUE)
        about.run()

    def on_properties1_activate(self, *args):
        if self.db:
            message = 'Current database: %s\n'\
                      '%d Cites - %d Routes\nAuthor: %s <%s>' % \
                      (self.db.filename, len(self.db.cities),
                       len(self.db.routes), self.db.author,
                       self.db.author_email)
        else:
            message = 'No database loaded.'
            
        dialog = GnomeOkDialog(message, self.appwin)
        dialog.run_and_close()

    def on_about1_close(self,*args):
        self.wtree.get_widget('aboutbox').hide()

    def on_addcity_clicked(self, *args):
        if not self.citywin: self.citywin = CityList(self.wtree)
        self.citywin.startup('', self.db, self.appendcity)

    def on_citylist_button_press_event(self, button, event, *args):
        if event.type == GDK._2BUTTON_PRESS and event.button == 1:
            self.on_changecity_clicked()

    def on_changecity_clicked(self, *args):
        if not self.citywin: self.citywin = CityList(self.wtree)
        pos = self.citylist.selection[0]
        self.citywin.startup('', self.db, self.changecity, pos)

    def appendcity(self, city, ind, *data):
        self.citylist.append([city])
        self.citylist.set_row_data(self.citylist.rows-1, ind)
        self.citylist.select_row(self.citylist.rows-1, 0)
        self.make_widgets_sensitive()

    def changecity(self, city, ind, pos):
        self.citylist.freeze()
        self.citylist.insert(pos, [city])
        self.citylist.set_row_data(pos, ind)
        self.citylist.remove(pos+1)
        self.citylist.select_row(pos, 0)
        self.citylist.thaw()

    def on_removecity_clicked(self, *args):
        pos = self.citylist.selection[0]
        self.citylist.remove(pos)
        rows = self.citylist.rows
        pos = min(rows-1, pos)
        self.citylist.select_row(pos, 0)
        self.make_widgets_sensitive()

    def on_up_clicked(self, *args):
        pos = self.citylist.selection[0]
        self.citylist.swap_rows(pos, pos-1)
        self.citylist.select_row(pos-1, 0)
        self.update_ud_widgets()

    def on_down_clicked(self, *args):
        pos = self.citylist.selection[0]
        self.citylist.swap_rows(pos, pos+1)
        self.citylist.select_row(pos+1, 0)
        self.update_ud_widgets()

    def update_ud_widgets(self):
        wlist = {'up': FALSE, 'down': FALSE,
                 'changecity' : TRUE, 'removecity' : TRUE}
        row = self.citylist.selection[0]
        rows = self.citylist.rows
        if row > 0:
            wlist['up'] = TRUE
        if row < rows-1:
            wlist['down'] = TRUE
        
        for widget, state in wlist.items():
            self.wtree.get_widget(widget).set_sensitive(state)

    def on_citylist_select_row(self, *args):
        self.update_ud_widgets()

    def on_citylist_unselect_row(self, *args):
        wlist = ['up', 'down', 'changecity', 'removecity']
        for widget in wlist:
            self.wtree.get_widget(widget).set_sensitive(FALSE)

    def on_filesel_cancel_clicked(self, *args):
        filewin = self.wtree.get_widget('fileselection1')
        filewin.hide()
        return TRUE
    
    def on_filesel_ok_clicked(self, *args):
        filewin = self.wtree.get_widget('fileselection1')
        filename = filewin.get_filename()
        filewin.hide()
        self.open_database(filename)

    def on_open1_activate(self, *args):
        filewin = self.wtree.get_widget('fileselection1')
        filewin.set_filename('/usr/share/routeplanner/')
        filewin.connect("delete_event", self.on_filesel_cancel_clicked)
        filewin.show()

    def on_plan_clicked(self, *args):
        db = self.db
        path = []
        for i in range(self.citylist.rows):
            cname = self.citylist.get_text(i, 0)
            cpos = self.citylist.get_row_data(i)
            info = db.cities[cpos]
            if path and info == path[-1]:
                dialog = GnomeErrorDialog('Same city appears twice in a '
                                          'row:\n'+cname+'\n'
                                          'Please fix this and plan again.',
                                          self.appwin)
                dialog.run_and_close()
                return
            path.append(info)

        methods = len(rpdbase.methods)
        displist = [None]*methods
        self.routes = [None]*methods
        progwin = ProgressWin(self.wtree, "Planning route")
        progwin.set_label('Planning')
        progwin.set_current(0)
        progwin.set_max(methods)
        progwin.show()

        for method in range(methods):
            progwin.set_label('Planning '+rpdbase.methods[method])
            progwin.set_current(method)
            trail = []
            for i in xrange(len(path)-1):
                trail.append( db.NewNavigate(path[i], path[i+1],
                                             method=method) )
            progwin.set_current(method+0.5)

            displist[method] = []
            for leg in trail:
                tlist = []
                entry = None
                for segment in leg:
                    preventry = entry
                    entry = ( segment[0].city[1-segment[1]],
                              segment[0].city[segment[1]],
                              segment[0].name,
                              segment[0].exits[segment[1]],
                              rpdbase.ConvertDistance(segment[0].distance,
                                                      segment[0].units,
                                                      self.units),
                              rpdbase.ConvertToTime(segment[0].speedcode,
                                                    segment[0].speed,
                                                    segment[0].distance,
                                                    segment[0].units,
                                                    db.units, db.speed),
                              rpdbase.Efficiency(segment[0].distance,
                                                 segment[0].units,
                                                 db.units, self.units,
                                                 segment[0].speedcode,
                                                 db.mpg, db.defmpg) )
                    if preventry:
                        if rpdbase.IsSameRoute(preventry[2], entry[2]):
                            del tlist[len(tlist)-1]
                            entry = (preventry[0], entry[1], preventry[2],
                                     entry[3], entry[4]+preventry[4],
                                     entry[5]+preventry[5],
                                     entry[6]+preventry[6])

                    tlist.append(entry)

                displist[method].append(tlist)

            del trail # Free some memory
            progwin.set_current(method+1)
        
        progwin.hide()

        planwin = self.wtree.get_widget('planwin')
        startloc = self.wtree.get_widget('startloc')
        endloc = self.wtree.get_widget('endloc')
        vialocs = self.wtree.get_widget('vialocs')

        start, end, via = path[0], path[-1], path[1:-1]
        startloc.set_text(str(start))
        endloc.set_text(str(end))
        if via:
            vialocs.set_text(string.join(map(str, via), '; '))
        else:
            vialocs.set_text('[none]')

        font = load_font("fixed")
        black = GdkColor(0,0,0)

        # Display our route
        if self.units == 'km':
            mpgunits = 'l'
        else:
            mpgunits = 'gal'

        widgets = PANES
        for j in range(methods):
            totals = (0,0,0)
            text = ""
            for i in range(len(displist[j])):
                text = text + 'From '+str(path[i])+' to '+str(path[i+1])+'\n'
                text = text + '-'*60 + '\n'

                leg = (0,0,0)
                for segment in displist[j][i]:
                    text = text + NicerTime(segment[5]) + ' ' + \
                           str(segment[2]) +' to ' + str(segment[1])
                    if segment[3]:
                        text = text + ' [Exit '+segment[3]+']'

                    text = text + ' (%.0f %s)\n' % (segment[4], self.units)
                    leg = leg[0] + segment[5], leg[1] + segment[4],\
                          leg[2] + segment[6]

                if len(displist[j]) > 1:
                    text = text + 'Leg totals: %.0f %s; %s (%.1f %s)\n\n' % (
                        leg[1], self.units, NicerTime(leg[0]), leg[2],
                        mpgunits)

                totals = (totals[0]+leg[0], totals[1]+leg[1], totals[2]+leg[2])

            text = text + 'Total route: %.0f %s; %s (%.1f %s)\n' % (
                totals[1], self.units, NicerTime(totals[0]),
                totals[2], mpgunits)
            
            route = self.wtree.get_widget(widgets[j])
            l = route.get_length()
            route.delete_text(0, l-1)
            route.insert(font, black, None, text)
            self.routes[j] = text
        
        planwin.connect("delete_event", self.close_route_win)
        planwin.show()

    def on_copy1_activate(self, *args):
        pane = self.wtree.get_widget('notebook1').get_current_page()
        widget = PANES[pane]
        self.wtree.get_widget(widget).copy_clipboard()

    def on_close1_activate(self, *args):
        self.close_route_win()

    def close_route_win(self, *args):
        self.wtree.get_widget('planwin').hide()
        return TRUE

    def make_widgets_sensitive(self):
        if not self.db:
            wlist = {'addcity' : FALSE,
                     'citylist' : FALSE,
                     'plan' : FALSE}
        elif self.citylist.rows > 1:
            wlist = {'addcity' : TRUE,
                     'citylist' : TRUE,
                     'plan' : TRUE}
        else:
            wlist = {'addcity' : TRUE,
                     'citylist' : TRUE,
                     'plan' : FALSE}

        for widget, state in wlist.items():
            self.wtree.get_widget(widget).set_sensitive(state)

    def open_database(self, filename):
        progwin = ProgressWin(self.wtree, "Loading "+filename)
        progwin.show()
        try:
            self.db = rpdbase.RPDatabase(filename, quiet=1,
                                         progressbar=progwin)
        except rpdbase.invalidformat:
            progwin.hide()
            error = GnomeErrorDialog('Unable to open\n'+filename, self.appwin)
            error.run_and_close()
            return

        self.make_widgets_sensitive()
        self.citylist.clear()
        progwin.hide()

def main(filename=''):
    app = Application(filename)
    mainloop()
